# perl interface for jags (just another Gibbs sampler)
# '17/07/05 by KGK

# geweke convergence diagnostic
package Makeshift::Jags::coda::geweke;

use Carp::Assert;
use Mouse;
#use Makeshift::Jags::coda::convergence;
extends 'Makeshift::Jags::coda::convergence';

# maximum Z-score for given p-value
my %Z_score = (0.10 => 1.65, 0.05 => 1.96, 0.01 => 2.58);

has 'method' => (is => 'ro', default => 'geweke');
has 'p' => (is => 'rw', default => 0.05);

__PACKAGE__->meta->make_immutable();

# read_log(file): read logfile and store result of monitored variables
# file: hash ref for handle-name of log file
sub read_log {
	my $self = shift;
	my $file = shift;
	my $log = $file->{name};
	my $fh = $file->{handle};

	assert($log, "log file must be specified");
	assert(-f $log, "$log is not a file");
	assert(-r $log, "$log is not readable");

	while (<$fh>) {
		next if /^Fraction/;
		next if /^$/;
		chomp;
		s/^\s+//;
		my @var = split;
		my @line = split /\s+/, <$fh>;
		shift @line unless $line[0];
		for (@line) {
			my $name = shift @var;
			push @{$self->names}, $name;
			$self->value->{$name} = $_;
		}
		last;
	}

	return $self;
}

# ok(val): convergence check for each parameter
# val: value of convergence diagnostic
# return value -> bool (OK/NG)
sub ok {
	my $self = shift;
	my $val = shift;

	return abs($val) <= $Z_score{$self->p};
}

# __PACKAGE__->meta->make_immutable();

1;
# end
