# perl interface for jags (just another Gibbs sampler)
# '17/07/05 by KGK

# set of variables in jags
package Makeshift::Jags::vars;

use Carp::Assert;
use Mouse;
use Makeshift::Jags::var;

has '_vars' => (is => 'ro', default => sub {{}});

__PACKAGE__->meta->make_immutable();

# set(var1 => val1, var2 => val2, ...): set variable value
# var: name of variable
# val: value of variable. For array ref, variable is treated as vector
sub set {
    my $self = shift;
	assert(@_ > 0, "(var => value) must be specified");
	assert(@_ % 2 == 0, "odd number args");

	my %arg = @_;

	for (keys %arg) {
		my $var = $self->_vars->{$_} // Makeshift::Jags::var->new(name => $_);
		$self->_vars->{$_} = $var->set($arg{$_});
	}

	return $self;
}

# get(var): get variable as Makeshift::Jags::var
# var: name of variable
sub get {
    my $self = shift;
	my $var = shift;

	assert($var, "variable name must be specified");

	return $self->_vars->{$var};
}

# del(var): delete variable
# var: name of variable
sub del {
    my $self = shift;
	my $var = shift;

	assert($var, "variable name must be specified");

	delete $self->_vars->{$_};

	return $self;
}

# save(fh): write R-style file for jags
# fh: file handle
sub save {
	my $self = shift;
	my $fh = shift;
	my @var = $self->get_array;

	assert(@var > 0, "value must be set before to(fh)");
	assert($fh, "file handle must be specified");

	for (@var) {
		print $fh $_->Rstyle, "\n";
	}

    return $self;
}

# get_array: return sorted array of Makeshift::Jags::var
sub get_array {
	my $self = shift;
	my @ret = ();

	for (sort keys %{$self->_vars}) {
		push @ret, $self->_vars->{$_};
	}

	return @ret;
}

1;
# end
